@@ -52,6 +52,8 @@ ctmm.fit <- function(data,CTMM=ctmm(),method="pHREML",COV=TRUE,control=list(),tr
   loglike.fn <- get.loglike(data)
 
   if(!is.null(control$message)) { message <- control$message }
+  # pass trace argument (demoted)
+  if(is.null(control$trace) && trace) { control$trace <- trace-1 }
 
   method <- match.arg(method,c("ML","pREML","pHREML","HREML","REML"))
   axes <- CTMM$axes
@@ -74,12 +76,17 @@ ctmm.fit <- function(data,CTMM=ctmm(),method="pHREML",COV=TRUE,control=list(),tr
     }
   }
 
-  # TODO CTPM LAG STUFFs
-
-  # standardize data for numerical stability
+  ### standardize data for numerical stability ###
+  DATA <- data[,axes,drop=FALSE]
+  # but apply link first
+  link <- get.link(CTMM)
+  log.like.link <- sum(log(abs(link$grad(DATA))))
+  DATA <- link$fn(DATA)
+  CTMM$link <- "identity" # does not need to be applied again in loglike function
   # pre-centering the data reduces later numerical error across models (tested)
-  SHIFT <- colMeans(get.telemetry(data,axes))
-  data[,axes] <- t( t(data[,axes,drop=FALSE]) - SHIFT )
+  SHIFT <- colMeans(DATA)
+  data[,axes] <- t( t(DATA) - SHIFT )
+  rm(DATA)
   # add mu.center back to the mean value after kalman filter / mean profiling
   # pre-standardizing the data should also help
   SCALE <- sqrt(mean(get.telemetry(data,axes)^2))
@@ -205,7 +212,7 @@ ctmm.fit <- function(data,CTMM=ctmm(),method="pHREML",COV=TRUE,control=list(),tr
   fn <- function(p,zero=0)
   {
     names(p) <- NAMES
-    p <- clean.parameters(p,profile=profile,linear.cov=linear.cov)
+    p <- clean.parameters(p,profile=profile,linear.cov=linear.cov,timelink=CTMM$timelink)
     CTMM <- set.parameters(CTMM,p,profile=profile,linear.cov=linear.cov)
 
     # negative log likelihood
@@ -213,7 +220,7 @@ ctmm.fit <- function(data,CTMM=ctmm(),method="pHREML",COV=TRUE,control=list(),tr
   }
 
   # shift parameters back near origin to prevent overflow in optimizer
-  reset <- function(p) { clean.parameters(p,profile=profile,linear.cov=FALSE) }
+  reset <- function(p) { clean.parameters(p,profile=profile,linear.cov=FALSE,timelink=CTMM$timelink) }
 
   # construct covariance matrix guess - must account for differences between storage and optimization representations
   # covariance <- function()
@@ -287,18 +294,31 @@ ctmm.fit <- function(data,CTMM=ctmm(),method="pHREML",COV=TRUE,control=list(),tr
   else ### all further cases require optimization ###
   {
     if(trace) { message("Maximizing likelihood.") }
+    # fix for bad starting conditions
+    if(class(data)[1]=="phylometry" && length(CTMM$tau)>1)
+    {
+      TEST <- fn(pars)
+      while(TEST>=Inf)
+      {
+        if(CTMM$tau[1]==CTMM$tau[2])
+        { pars['tau'] <- pars['tau']/2 }
+        else
+        { pars['tau velocity'] <- pars['tau velocity']/2 }
+        TEST <- fn(pars)
+      }
+    }
     # control$covariance <- covariance() - parameter storage and optimization representations can differ
     control$parscale <- parscale
     control$zero <- TRUE
     RESULT <- optimizer(par=pars,fn=fn,method=op.method,lower=lower,upper=upper,period=period,reset=reset,control=control)
-    pars <- clean.parameters(RESULT$par,profile=profile)
+    pars <- clean.parameters(RESULT$par,profile=profile,timelink=CTMM$timelink)
     # copy over hessian from fit to COV.init ?
 
     # write best estimates over initial guess
     store.pars <- function(pars,profile=profile,finish=TRUE)
     {
       names(pars) <- NAMES
-      pars <- clean.parameters(pars,profile=profile,linear.cov=linear.cov)
+      pars <- clean.parameters(pars,profile=profile,linear.cov=linear.cov,timelink=CTMM$timelink)
 
       CTMM <- set.parameters(CTMM,pars,profile=profile,linear.cov=linear.cov)
 
@@ -379,7 +399,7 @@ ctmm.fit <- function(data,CTMM=ctmm(),method="pHREML",COV=TRUE,control=list(),tr
         profile <- FALSE
         # this can still fail if parameters are crazy
         TEST <- store.pars(pars,profile=FALSE,finish=TRUE)
-        if(class(TEST)[1]=="ctmm")
+        if(class(TEST)[1]=="ctmm" && !is.na(TEST$loglike) && TEST$loglike>-Inf)
         {
           CTMM <- TEST
           linear.cov <- FALSE
@@ -393,7 +413,25 @@ ctmm.fit <- function(data,CTMM=ctmm(),method="pHREML",COV=TRUE,control=list(),tr
     # revert to ML/HREML if pREML step failed
     if(method %in% c("pREML","pHREML") && PREML.FAIL)
     {
-      warning("pREML failure: indefinite ML Hessian or divergent REML gradient.")
+      # shouldn't need to warn if using ctmm.select
+      WARN <- TRUE
+      N <- sys.nframe()
+      if(N>=2)
+      {
+        for(i in 2:N)
+        {
+          CALL <- deparse(sys.call(-i))[1]
+          CALL <- grepl("ctmm.select",CALL) || grepl("cv.like",CALL) || grepl("ctmm.boot",CALL)
+          if(CALL)
+          {
+            WARN <- FALSE
+            break
+          }
+        }
+      }
+      # warn if weren't using ctmm.select
+      if(WARN) { warning("pREML failure: indefinite ML Hessian or divergent REML gradient.") }
+
       if(method=='pREML') { method <- 'ML' }
       else if(method=='pHREML') { method <- 'HREML' }
     }
@@ -415,7 +453,7 @@ ctmm.fit <- function(data,CTMM=ctmm(),method="pHREML",COV=TRUE,control=list(),tr
         control$parscale <- parscale
         control$zero <- TRUE
         RESULT <- optimizer(par=pars,fn=fn,method=op.method,lower=lower,upper=upper,period=period,reset=reset,control=control)
-        pars <- clean.parameters(RESULT$par)
+        pars <- clean.parameters(RESULT$par,timelink=CTMM$timelink)
       }
 
       # includes free profile
@@ -457,6 +495,8 @@ ctmm.fit <- function(data,CTMM=ctmm(),method="pHREML",COV=TRUE,control=list(),tr
   CTMM$features <- NAMES # store all auto-covariance features
 
   # calculate AIC,AICc,BIC,MSPE,...
+  CTMM$link <- link$name
+  CTMM$loglike <- CTMM$loglike + log.like.link
   CTMM <- ic.ctmm(CTMM,n)
 
   # would be temporary ML COV for pREML/pHREML
@@ -464,6 +504,8 @@ ctmm.fit <- function(data,CTMM=ctmm(),method="pHREML",COV=TRUE,control=list(),tr
 
   if(method %in% c('pREML','pHREML','HREML') && !is.null(MLE))
   {
+    MLE$link <- link$name
+    MLE$loglike <- MLE$loglike + log.like.link
     # calculate checksum
     MLE$checksum <- digest::digest(CTMM,algo="md5")
     CTMM$MLE <- MLE
@@ -493,6 +535,7 @@ ic.ctmm <- function(CTMM,n)
     k.mean <- k.mean - 1
     n <- n - 1
   }
+  if(!length(k.mean)) { k.mean <- 0 } # failed fit (bad data or bad parameters)
   k <- nu + q*k.mean
 
   CTMM$AIC <- 2*k - 2*CTMM$loglike
@@ -574,6 +617,7 @@ ic.ctmm <- function(CTMM,n)
 ###################
 ctmm.guess <- function(data,CTMM=ctmm(),variogram=NULL,name="GUESS",interactive=TRUE)
 {
+  if(class(data)[1]=="list") { stop("ctmm.guess needs to be run individually.") }
   # use intended axes
   if(is.null(variogram)) { variogram = variogram(data,axes=CTMM$axes) }
   else { CTMM$axes <- attr(variogram,"info")$axes }
