@@ -368,107 +368,6 @@ fill.data <- function(data,CTMM=ctmm(tau=Inf),verbose=FALSE,t=NULL,dt=NULL,res=1
 }
 
 
-#################################
-# Kriged Kernel Density Estimate
-# H is your additional smoothing bandwidth matrix (zero by default)
-# resolution is the number of kriged locations per median step
-# cor.min is roughly the correlation required between locations to bridge them
-# dt.max is (alternatively) the maximum gap allowed between locations to bridge them
-#################################
-occurrence <- function(data,CTMM,H=0,res.time=10,res.space=10,grid=NULL,cor.min=0.05,dt.max=NULL,buffer=TRUE)
-{
-  validate.grid(data,grid)
-
-  axes <- CTMM$axes
-  CTMM0 <- CTMM
-  dt <- stats::median(diff(data$t))
-  if(is.null(dt.max)) { dt.max <- dt }
-
-  if(length(H)==1) { H <- diag(H,2) }
-
-  info <- attr(data,"info")
-  SIGMA <- CTMM$sigma # diffusion matrix for later
-  CTMM <- ctmm.prepare(data,CTMM,precompute=FALSE) # not the final t for calculating u
-  error <- get.error(data,CTMM,circle=TRUE)
-  MIN.ERR <- min(error) # Fix something here?
-
-  # format data to be relatively evenly spaced with missing observations
-  data <- fill.data(data,CTMM,verbose=TRUE,res=res.time,cor.min=cor.min,dt.max=dt.max,buffer=buffer)
-  t.grid <- data$t.grid
-  dt.grid <- data$dt.grid
-  w.grid <- data$w.grid
-  data <- data$data
-
-  # calculate trend
-  drift <- get(CTMM0$mean)
-  drift <- drift(data$t,CTMM0) %*% CTMM0$mu
-
-  # detrend for smoothing - retrend later
-  z <- get.telemetry(data,axes=axes)
-  data[,axes] <- z - drift
-
-  # smooth mean-zero data # run through smoother to get
-  state <- smoother(data,CTMM,smooth=TRUE)
-
-  # skip repeated timestamps in data (full information retained)
-  KEEP <- !data$skip
-  data <- data[KEEP,]
-  state$R <- state$R[KEEP,,drop=FALSE]
-  state$COV <- state$COV[KEEP,,,drop=FALSE]
-  if(length(CTMM$tau)>1) { state$V <- state$V[KEEP,,drop=FALSE] }
-  drift <- drift[KEEP,,drop=FALSE]
-
-  # detrend for simulation - retrend later
-  state$R <- state$R + drift
-
-  # evenly sampled subset: data points (bridge ends) may be counted twice and weighted half
-  GRID <- c( which(data$t %in% t.grid) , which(data$t %in% t.grid[which(diff(t.grid)==0)]) )
-  GRID <- sort.int(GRID)
-  # GRID <- data$t %in% t.grid
-
-  # t <- state$t[GRID]
-  R <- state$R[GRID,]
-  COV <- state$COV[GRID,,]
-  n <- length(R[,1])
-
-  # continuous velocities will give us more information to use
-  if(length(CTMM$tau)>1)
-  { V <- state$V[GRID,] }
-  else # null velocity data otherwise
-  { V <- array(0,c(n,2)) }
-
-  # fake data
-  data <- data.frame(x=R[,1],y=R[,2])
-
-  # some covariances are slightly negative due to roundoff error
-  # so here I clamp the negative singular values to zero
-  COV <- vapply(1:n,function(i){ PDclamp(COV[i,,]) },COV[1,,]) # (d,d,n)
-  COV <- aperm(COV,perm=c(3,1,2)) # (n,d,d)
-
-  # uncertainties/bandwidths for this data
-  H <- vapply(1:n,function(i){ H + COV[i,,] + dt.grid[i]^2/12*(V[i,] %o% V[i,]) },H) # (2,2,n)
-  H <- aperm(H,c(3,1,2)) # (n,2,2)
-
-  # estimate size of data blob
-  dr <- diag(SIGMA)
-  if(CTMM$range && length(CTMM$tau)) { dr <- dr/CTMM$tau[1] }
-  # prefactors from mid-bridge variance
-  if(length(CTMM$tau)==1) #BM/OU
-  { dr <- dt/4 * dr }
-  else if(length(CTMM$tau)==2) #IOU/OUF
-  { dr <- dt^2/24 * dr/CTMM$tau[2] }
-
-  if(CTMM$error){ dr <- dr + MIN.ERR }
-  dr <- sqrt(dr)
-
-  # using the same data format as AKDE, but with only the ML estimate (alpha=1)
-  KDE <- kde(data,H=H,W=w.grid,dr=dr/res.space,grid=grid)
-  KDE$H <- diag(0,2)
-  KDE <- new.UD(KDE,info=info,type='occurrence',CTMM=CTMM0)
-  return(KDE)
-}
-
-
 ##############################################
 # SIMULATE DATA over time array t
 simulate.ctmm <- function(object,nsim=1,seed=NULL,data=NULL,VMM=NULL,t=NULL,dt=NULL,res=1,complete=FALSE,precompute=FALSE,...)
@@ -573,7 +472,7 @@ simulate.ctmm <- function(object,nsim=1,seed=NULL,data=NULL,VMM=NULL,t=NULL,dt=N
   } # conditional simulation
   else # Gaussian simulation not conditioned off of any data
   {
-    STUFF <- c('Green','Sigma','error','object','mu','Lambda','n','K','z','v','circle','R')
+    STUFF <- c('Green','Sigma','error','ELLIPSE','object','mu','Lambda','n','K','z','v','circle','R')
     if(precompute>=0)
     {
       if(is.null(data))
@@ -591,6 +490,7 @@ simulate.ctmm <- function(object,nsim=1,seed=NULL,data=NULL,VMM=NULL,t=NULL,dt=N
       {
         z <- get.telemetry(data)
         v <- NULL
+        Green <- Sigma <- mu <- Lambda <- K <- circle <- R <- NULL # make sure get() doesn't fail
       }
       else # have model
       {
@@ -690,6 +590,8 @@ simulate.ctmm <- function(object,nsim=1,seed=NULL,data=NULL,VMM=NULL,t=NULL,dt=N
         colnames(v) <- paste0("v",axes)
       }
     } # end if(!is.null(object))
+    else # no process variance
+    { z <- (object$mean.vec %*% mu) }
 
     # throw in error
     if(any(object$error>0))
@@ -721,7 +623,7 @@ simulate.ctmm <- function(object,nsim=1,seed=NULL,data=NULL,VMM=NULL,t=NULL,dt=N
   data <- new.telemetry(data,info=info)
   if(complete)
   {
-    if(axes=='z') { stop("(x,y) locations must also be simulated for complete=TRUE.") }
+    if(all(axes=='z')) { stop("(x,y) locations must also be simulated for complete=TRUE.") }
     data <- pseudonymize(data,tz=info$timezone,proj=info$projection,origin=EPOCH)
   }
 
@@ -805,7 +707,7 @@ predict.ctmm <- function(object,data=NULL,VMM=NULL,t=NULL,dt=NULL,res=1,complete
     sigma <- methods::getDataPart(object$sigma)
     if(length(axes)==1 || object$isotropic)
     {
-      sigma <- mean(diag(sigma,length(axes)))
+      if(length(sigma)>1) { sigma <- mean(diag(sigma,length(axes))) }
       data[[DOP.LIST[[DOP]]$VAR]] <- sigma
       if(length(object$tau)>1) { data[[paste0("VAR.v",axes)]] <- sigma/prod(object$tau) }
     }
@@ -892,7 +794,7 @@ predict.ctmm <- function(object,data=NULL,VMM=NULL,t=NULL,dt=NULL,res=1,complete
   data <- new.telemetry(data,info=info)
   if(complete)
   {
-    if(axes=='z') { stop("(x,y) locations must also be predicted for complete=TRUE.") }
+    if(all(axes=='z')) { stop("(x,y) locations must also be predicted for complete=TRUE.") }
     data <- pseudonymize(data,tz=info$timezone,proj=info$projection,origin=EPOCH)
   }
 
